# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	modules/NovellDns.ycp
# Package:	Configuration of novell-dns
# Summary:	NovellDns settings, input and output functions
# Authors:	Mark Clark <mclark@novell.com>
#
# $Id: NovellDns.ycp 27914 2006-02-13 14:32:08Z locilka $
#
# Representation of the configuration of novell-dns.
# Input and output routines.
require "yast"

module Yast
  class NovellDnsClass < Module
    def main
      textdomain "novell-dns"


      Yast.import "Popup"
      Yast.import "Service"
      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Misc"
      Yast.import "Summary"
      Yast.import "Edirectory"
      Yast.import "Mode"
      Yast.import "NovellUtils"
      Yast.import "NovellValidators"
      Yast.import "NovellLDAP"
      Yast.import "Message"
      Yast.import "NovellPatterns"
      Yast.import "NovellAddonInstall"
      Yast.import "Stage"
      Yast.import "Wizard"
      @auto_clone = false

      @edir_service_configured = false

      @xad_packages_installed = false
      @xadMode = false

      @xadTestMode = false

      @schemaLogPath = "/var/opt/novell/eDirectory/log/oes_schema.log"

      # Settings Map
      @Settings = {}
      @oldSettings = {}

      # Data was modified?
      @modified = false
      @upgrade_from = ""

      @proposal_valid = false

      # Are we in reinstall mode?
      @reinstallMode = false

      # Are we in upgrade mode?
      @upgradeMode = false

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false
      @globals_configured = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")

      @scr_conf_path = path(".sysconfig.novell-dns")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    # insufficient_rights_error()
    #
    # Checks a string for output indicating insufficient rights.
    def insufficient_rights_error(msg)
      return false if msg == nil || msg == ""

      idx = Builtins.search(msg, "Insufficient access")
      return true if Ops.greater_or_equal(idx, 0)

      idx = Builtins.search(msg, "no access (-672)")
      return true if Ops.greater_or_equal(idx, 0)

      false
    end

    def GetTreeType
      treeType = ""
      if Mode.installation && Stage.normal && @edir_service_configured
        treeType = Convert.to_string(
          SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))
        ) # next try reading the edir conf file
        Builtins.y2milestone(
          "novell-dns.GetTreeType (post install):tree type from edir conf file:%1",
          treeType
        )
      else
        treeType = Ops.get_string(
          Edirectory.EdirSettings,
          "CONFIG_EDIR_TREE_TYPE",
          ""
        )
        Builtins.y2milestone(
          "novell-dns.GetTreeType:tree type from eDir map:%1",
          treeType
        )
        if treeType == nil || treeType == ""
          treeType = Convert.to_string(
            SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))
          ) # next try reading the edir conf file
          Builtins.y2milestone(
            "novell-dns.GetTreeType:tree type from edir conf file:%1",
            treeType
          )
        end
      end
      treeType = "" if treeType == nil

      Builtins.tolower(treeType)
    end
    def ReadOption(optname, replace)
      optvalue = Convert.to_string(
        SCR.Read(Ops.add(@scr_conf_path, Builtins.topath(optname)))
      )

      optvalue = "" if optvalue == nil
      optvalue = replace if optvalue == "" && replace != nil

      optvalue
    end

    def ServerIsDsfwServer(ip)
      retCode = false

      cmd = Builtins.sformat(
        "/usr/bin/ldapsearch -x -H ldaps://%1:636 -s base -b \"\" | grep 2.16.840.1.113719.1.513.7.1 > /dev/null",
        ip
      )
      Builtins.y2milestone("NovellDns.ServerIsDsfwServer:executing %1", cmd)
      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      retCode = true if Ops.get_integer(ret, "exit", -1) == 0 # server is a dsfw server

      Builtins.y2milestone(
        "NovellDns.ServerIsDsfwServer:returned %1",
        Ops.get_integer(ret, "exit", -1)
      )
      retCode
    end

    def GetContextInfo(dnsIP)
      # perl /opt/novell/named/bin/dns-getctx.pl -s 164.99.100.123 -p 636 -l 1 -a 'cn=admin,o=novell' -d 164.99.100.123  -u 2

      contexts = []

      # DNS wants the remote server if availble
      server_list = NovellLDAP.GetRemoteServers(nil)
      keyIP = ""
      if Ops.greater_than(Builtins.size(server_list), 0)
        keyIP = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
      else
        server_list = NovellLDAP.GetLocalServers(nil)
        keyIP = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
      end


      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
      )
      primary_server = NovellLDAP.GetAddressServer(keyIP, server_list)

      ldapIP = Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", "")
      remoteServerIsDsfw = ServerIsDsfwServer(ldapIP)

      ldapPort = Builtins.sformat(
        "%1",
        Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636)
      )
      ldapPort = "1636" if remoteServerIsDsfw

      useSSL = "1"
      adminDn = NovellUtils.Nds2Ldap(
        Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      )
      adminPassword = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )

      cmd = Builtins.sformat(
        "perl /opt/novell/named/bin/dns-getctx.pl -s %1 -p %2 -l %3 -a '%4' -d %5 -u 1",
        ldapIP,
        ldapPort,
        useSSL,
        adminDn,
        dnsIP
      )

      if @xadTestMode
        Builtins.y2milestone(
          "NovellDns.GetContextInfo:executing: %1",
          Ops.add(Ops.add(Ops.add("OES_DATA='", adminPassword), "' "), cmd)
        )
      else
        Builtins.y2milestone("NovellDns.GetContextInfo:executing: %1", cmd)
      end

      rmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          Ops.add(Ops.add(Ops.add("OES_DATA='", adminPassword), "' "), cmd)
        )
      )
      if Ops.get_integer(rmap, "exit", -1) != 0
        Builtins.y2milestone(
          "novelldns.GetContextInfo:could not get DNS contexts. Error: %1",
          Ops.get_integer(rmap, "exit", -1)
        )
        contexts = nil
      else
        contextFile = "/opt/novell/named/bin/dns-ctx"
        if Ops.greater_than(SCR.Read(path(".target.size"), contextFile), 0)
          contextData = Convert.to_string(
            SCR.Read(path(".target.string"), contextFile)
          )
          contexts = Builtins.splitstring(contextData, "\n")
          Builtins.y2milestone(
            "novelldns.GetContextInfo:contexts: %1",
            contexts
          )
        else
          Builtins.y2milestone(
            "novelldns.GetContextInfo:script call to dns-getctx did not return an error, but the file '/opt/novell/named/bin/dns-ctx' is missing."
          )
          contexts = nil
        end
      end

      deep_copy(contexts)
    end

    def ValidateCreds(userName, userPassword)
      do_continue = false

      userName = NovellUtils.Ldap2Nds(userName)

      server_list = NovellLDAP.GetRemoteServers(nil)
      keyIP = ""
      if Ops.greater_than(Builtins.size(server_list), 0)
        keyIP = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
      else
        server_list = NovellLDAP.GetLocalServers(nil)
        keyIP = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
      end


      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
      )
      primary_server = NovellLDAP.GetAddressServer(keyIP, server_list)

      ldapIP = Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", "")

      ldapPort = Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636)

      val = NovellValidators.ValidateCredentials(
        userName,
        userPassword,
        ldapIP,
        ldapPort,
        true
      )
      Builtins.y2milestone(
        "NovellDns:validating creds. name %1, pass ****, ip %2, port %3",
        userName,
        ldapIP,
        ldapPort
      )
      if val != 0
        Builtins.y2milestone("ValidateCredentials() returned %1", val)
        if val == -666
          err = Edirectory.ldap_bind_no_response
          yes = Popup.YesNoHeadline(Edirectory.ldap_secure_bind_timeout, err)
          do_continue = true if !yes
        else
          err = Builtins.sformat(
            Edirectory.validateCredsWarning,
            ldapIP,
            userName,
            ldapPort
          )
          yes = Popup.YesNoHeadline(Edirectory.credentialsFailedWarning, err)
          do_continue = true if !yes
        end
      end

      do_continue
    end

    def UserExists(user)
      ret = false

      # perl /opt/novell/named/bin/dnsdhcp-search-proxy-user.pl -s 137.65.59.48 -p 636 -l 1 -D "cn=admin,o=novell" -u "cn=myuser,o=novell" -t 15

      user = NovellUtils.Nds2Ldap(user)

      # DNS wants the remote server if availble
      server_list = NovellLDAP.GetRemoteServers(nil)
      keyIP = ""
      if Ops.greater_than(Builtins.size(server_list), 0)
        keyIP = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
      else
        server_list = NovellLDAP.GetLocalServers(nil)
        keyIP = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
      end

      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
      )
      primary_server = NovellLDAP.GetAddressServer(keyIP, server_list)

      ldapIP = Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", "")
      remoteServerIsDsfw = ServerIsDsfwServer(ldapIP)

      ldapPort = Builtins.sformat(
        "%1",
        Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636)
      )
      ldapPort = "1636" if remoteServerIsDsfw

      useSSL = "1"
      adminDn = NovellUtils.Nds2Ldap(
        Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      )
      adminPassword = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )

      cmd = Builtins.sformat(
        "perl /opt/novell/named/bin/dnsdhcp-search-proxy-user.pl -s %1 -p %2 -l %3 -D '%4' -u '%5' -t 15",
        ldapIP,
        ldapPort,
        useSSL,
        adminDn,
        user
      )

      if @xadTestMode
        Builtins.y2milestone(
          "NovellDns.UserExists:executing: %1",
          Ops.add(Ops.add(Ops.add("OES_DATA='", adminPassword), "' "), cmd)
        )
      else
        Builtins.y2milestone("NovellDns.UserExists:executing: %1", cmd)
      end

      rmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          Ops.add(Ops.add(Ops.add("OES_DATA='", adminPassword), "' "), cmd)
        )
      )
      ret = true if Ops.get_integer(rmap, "exit", -1) == 0 # proxy user exists

      ret
    end

    # Read all novell-dns settings
    # @return true on success
    def Read
      # NovellDns read dialog caption
      caption = _("Initializing novell-dns Configuration")

      # TODO FIXME Set the right number of stages
      steps = 4

      sl = 500
      Builtins.sleep(sl)


      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Read OES DNS settings"),
          _("Checking dependencies")
        ],
        [
          # Progress step 1/1
          _("Reading OES DNS settings..."),
          _("Checking dependencies"),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read current settings
      return false if Abort()
      Progress.NextStage

      # First, get the default LDAP/eDir data
      progress_orig = Progress.set(false)
      NovellLDAP.Read
      Progress.set(progress_orig)

      # See if XAD is installed. If so, default the data.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "NovellDns:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end
      Builtins.y2milestone("NovellDns:xadMode:%1", @xadMode)

      if @xadMode
        Builtins.y2milestone(
          "NovellDns.Read:Leaving Read since we are in XAD mode."
        )
        return true
      end

      @upgrade_from = NovellUtils.UpgradeIsFrom
      if (Mode.update || Mode.autoupgrade) &&  @upgrade_from != ""
        @oldSettings = NovellUtils.GetOldSysconfigMap("novell-dns")
        @upgradeMode = true
      end

      # determine if DNS Services configuration already exists
      service_configured = Builtins.tolower(
        ReadOption("SERVICE_CONFIGURED", "no")
      )
      service_previously_configured = Builtins.tolower(
        ReadOption("SERVICE_PREVIOUSLY_CONFIGURED", "no")
      )

      if @upgradeMode
        Ops.set(
          @Settings,
          "CONFIG_DNS_LDAP_SERVER",
          Ops.get_string(@oldSettings, "CONFIG_DNS_LDAP_SERVER", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_LDAP_BASEDN",
          Ops.get_string(@oldSettings, "CONFIG_DNS_LDAP_BASEDN", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_SERVER_CONTEXT",
          Ops.get_string(@oldSettings, "CONFIG_DNS_SERVER_CONTEXT", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_LOCATOR_CONTEXT",
          Ops.get_string(@oldSettings, "CONFIG_DNS_LOCATOR_CONTEXT", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_GROUP_CONTEXT",
          Ops.get_string(@oldSettings, "CONFIG_DNS_GROUP_CONTEXT", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_USE_SECURE_PORT",
          Ops.get_string(@oldSettings, "CONFIG_DNS_USE_SECURE_PORT", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_LDAP_USER_CONTEXT",
          Ops.get_string(@oldSettings, "CONFIG_DNS_LDAP_USER_CONTEXT", "")
        )
        Ops.set(@Settings, "CONFIG_DNS_CREATE_SERVER_OBJECT", "no")
        Ops.set(@Settings, "CONFIG_DNS_STORE_CREDENTIAL_IN_OCS", "yes")
        Ops.set(
          @Settings,
          "CONFIG_DNS_HOST_NAME",
          Ops.get_string(@oldSettings, "CONFIG_DNS_HOST_NAME", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_DOMAIN_NAME",
          Ops.get_string(@oldSettings, "CONFIG_DNS_DOMAIN_NAME", "")
        )

        if NovellUtils.Nds2Ldap(
            Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", "")
          ) ==
            NovellUtils.Nds2Ldap(
              Ops.get_string(
                NovellLDAP.Settings,
                "CONFIG_LDAP_ADMIN_CONTEXT",
                ""
              )
            )
          Ops.set(
            @Settings,
            "CONFIG_DNS_LDAP_USER_PASSWORD",
            Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_ADMIN_PASSWORD",
              ""
            )
          )
        end
      else
        # Next get Novell DNS Services data
        servername = ReadOption("CONFIG_DNS_LDAP_SERVER", "")
        if servername == ""
          server_list = NovellLDAP.GetLocalServers(nil)
          if Ops.greater_than(Builtins.size(server_list), 0)
            servername = Ops.get_string(
              Ops.get(server_list, 0, {}),
              "CONFIG_LDAP_SERVER_IP",
              ""
            )
          else
            server_list = NovellLDAP.GetRemoteServers(nil)
            servername = Ops.get_string(
              Ops.get(server_list, 0, {}),
              "CONFIG_LDAP_SERVER_IP",
              ""
            )
          end
        end

        full_admin_context = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        if @xadMode
          xad_tree_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "NovellDns:XAD_TREE_ADMIN_CONTEXT:%1",
            xad_tree_admin_context
          )
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            full_admin_context = NovellUtils.Nds2Ldap(xad_tree_admin_context)
          end
        end
        Builtins.y2milestone("NovellDns:admin:%1", full_admin_context)

        ldapadmincontext = NovellUtils.GetContextNDS(full_admin_context)

        info = NovellAddonInstall.GetProductInfo("edirectory")
        if Ops.get_boolean(info, "SERVICE_CONFIGURED", false) == true
          @edir_service_configured = true
        else
          @edir_service_configured = false
        end

        edirservercontext = ""
        if Mode.installation && Stage.normal && @edir_service_configured
          edirservercontext = Misc.SysconfigRead(
            path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"),
            ""
          )
          Builtins.y2milestone(
            "NovellDns:server context from edir file (post install):%1",
            edirservercontext
          )
        else
          edirservercontext = Ops.get_string(
            Edirectory.EdirSettings,
            "CONFIG_EDIR_SERVER_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "NovellDns:server context from edir map:%1",
            edirservercontext
          )
          if edirservercontext == nil || edirservercontext == ""
            edirservercontext = Misc.SysconfigRead(
              path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"),
              ""
            )
            Builtins.y2milestone(
              "NovellDns:server context from edir file:%1",
              edirservercontext
            )
          end
        end
        usedLdapUser = false
        if edirservercontext == nil || edirservercontext == ""
          edirservercontext = ldapadmincontext
          usedLdapUser = true
        end

        edirservercontext = NovellUtils.Nds2Ldap(edirservercontext)

        Ops.set(@Settings, "CONFIG_DNS_LDAP_SERVER", servername)
        Ops.set(
          @Settings,
          "CONFIG_DNS_SERVER_CONTEXT",
          ReadOption("CONFIG_DNS_SERVER_CONTEXT", edirservercontext)
        )

        Ops.set(
          @Settings,
          "CONFIG_DNS_LOCATOR_CONTEXT",
          ReadOption("CONFIG_DNS_LOCATOR_CONTEXT", edirservercontext)
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_LDAP_BASEDN",
          ReadOption("CONFIG_DNS_LDAP_BASEDN", edirservercontext)
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_GROUP_CONTEXT",
          ReadOption("CONFIG_DNS_GROUP_CONTEXT", edirservercontext)
        )

        Ops.set(
          @Settings,
          "CONFIG_DNS_USE_SECURE_PORT",
          Builtins.tolower(ReadOption("CONFIG_DNS_USE_SECURE_PORT", "yes"))
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_LDAP_USER_CONTEXT",
          ReadOption("CONFIG_DNS_LDAP_USER_CONTEXT", full_admin_context)
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_CREATE_SERVER_OBJECT",
          Builtins.tolower(ReadOption("CONFIG_DNS_CREATE_SERVER_OBJECT", "no"))
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_HOST_NAME",
          ReadOption("CONFIG_DNS_HOST_NAME", NovellUtils.GetHostname)
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_DOMAIN_NAME",
          ReadOption("CONFIG_DNS_DOMAIN_NAME", NovellUtils.GetDomainName)
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_EXISTING_DNS_IP",
          ReadOption("CONFIG_DNS_EXISTING_DNS_IP", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_DNS_STORE_CREDENTIAL_IN_OCS",
          ReadOption("CONFIG_DNS_STORE_CREDENTIAL_IN_OCS", "yes")
        )

        useCommonProxy = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_DO_PROXY",
          ""
        )
        Builtins.y2milestone(
          "NovellDns.Read:useCommonProxy: %1",
          useCommonProxy
        )

        commonProxyName = ""
        if useCommonProxy == "yes"
          commonProxyName = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
          )
          Builtins.y2milestone(
            "NovellDns.Read:commonProxyName: '%1'",
            commonProxyName
          )
          commonProxyPassword = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_PROXY_PASSWORD",
            ""
          )
        end

        if service_configured != "yes" # new install
          if service_previously_configured != "yes"
            @reinstallMode = false
          else
            @reinstallMode = true
          end
          Ops.set(
            @Settings,
            "CONFIG_DNS_LDAP_USER_PASSWORD",
            Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_ADMIN_PASSWORD",
              ""
            )
          )
          if Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_PASSWORD", "") == nil
            Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", "")
          end

          if useCommonProxy == "yes"
            Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", commonProxyName)
            if commonProxyPassword != ""
              Ops.set(
                @Settings,
                "CONFIG_DNS_LDAP_USER_PASSWORD",
                commonProxyPassword
              )
            else
              Ops.set(
                @Settings,
                "CONFIG_DNS_LDAP_USER_PASSWORD",
                NovellUtils.RetrieveProxyCredentials("password")
              )
            end
            Ops.set(@Settings, "CONFIG_DNS_COMMON_PROXY", commonProxyName)
            Ops.set(
              @Settings,
              "CONFIG_DNS_COMMON_PROXY_PASSWORD",
              Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_PASSWORD", "")
            )
          end # the config file exists, so use the data. Reconfigufe mode.
        else
          @reinstallMode = true
          if useCommonProxy == "yes"
            if commonProxyPassword == ""
              commonProxyPassword = NovellUtils.RetrieveProxyCredentials(
                "password"
              )
            end

            if NovellUtils.Nds2Ldap(
                Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", "")
              ) ==
                NovellUtils.Nds2Ldap(commonProxyName)
              Ops.set(
                @Settings,
                "CONFIG_DNS_LDAP_USER_PASSWORD",
                commonProxyPassword
              )
            end
            Ops.set(@Settings, "CONFIG_DNS_COMMON_PROXY", commonProxyName)
            Ops.set(
              @Settings,
              "CONFIG_DNS_COMMON_PROXY_PASSWORD",
              commonProxyPassword
            )
          else
            Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_PASSWORD", "") # We always ask for this and never store it in /etc/sysconfig
          end
        end
      end
      @globals_configured = true
      Builtins.y2debug("Novell DNS READ Complete: %1", @Settings)

      # Error message
      if false
        Report.Error(_("Cannot read current OES DNS Services settings."))
      end
      Builtins.sleep(sl)

      # check dependencies
      return false if Abort()
      Progress.NextStage

      return false if Abort()
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end

    # Write all novell-dns settings
    # @return true on success
    def Write
      # Novell DNS Services write dialog caption
      pattern = "novell-dns"
      Wizard.DisableAbortButton
      if NovellPatterns.IsPatternInstalled(pattern) == false
        Builtins.y2milestone(
          "The required pattern, %1, is not installed.",
          pattern
        )
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration aborted. Install the required %1 pattern and try again."
              ),
              pattern
            )
          )
        end

        return true
      end

      if !Ops.get(NovellAddonInstall.products_to_configure, "novell-dns", true)
        Builtins.y2milestone("Novell DNS -- configure later")
        return true
      end

      abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if abortInstall == "yes"
        Builtins.y2milestone(
          "Novell DNS:setting ABORT_OES_INSTALL is set to 'yes'."
        )
        return true
      end

      # See if XAD is installed. If so, default the data.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "NovellDns:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end
      Builtins.y2milestone("NovellDns:xadMode:%1", @xadMode)

      if @xadMode
        Builtins.y2milestone(
          "NovellDns.Write:Leaving Read since we are in XAD mode."
        )
        return true
      end
      sl = 500
      retval = {}

      if Mode.autoinst
        commonProxyName = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
        )
        if commonProxyName != ""
          Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", commonProxyName)
        else
          Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", NovellUtils.RetrieveProxyCredentials("username"))
        end
        commonProxyPassword = Ops.get_string(NovellLDAP.Settings,"CONFIG_LDAP_PROXY_PASSWORD","")
        if commonProxyPassword != ""
          Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_PASSWORD", commonProxyPassword)
        else
          Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_PASSWORD", NovellUtils.RetrieveProxyCredentials("password"))
        end
      end

      if !@upgradeMode
        caption = _("Saving OES DNS Services Configuration")

        steps = 1


        Builtins.sleep(sl)

        # We do not set help text here, because it was set outside
        Progress.New(
          caption,
          " ",
          steps,
          [
            # Progress stage 1/1
            _("Write OES DNS Services settings")
          ],
          [
            # Progress step 1/1
            _("Writing OES DNS Services settings..."),
            # Progress finished
            _("Finished")
          ],
          ""
        )

        # write settings
        return false if Abort()
        Progress.NextStage

        SCR.Write(path(".sysconfig.novell-dns.SERVICE_CONFIGURED"), "no")

        reinstallStr = ""
        reinstallStr = "REINSTALL='1' " if @reinstallMode

        upgradeStr = ""
        upgradeStr = "UPGRADE='1' " if @upgradeMode

        cmd = ""
        rval = 0

        # determine IP addresses
        server_list = Ops.get_list(
          NovellLDAP.Settings,
          "CONFIG_LDAP_SERVERS_LIST",
          []
        )
        primary_server = NovellLDAP.GetAddressServer(
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_SERVER", ""),
          server_list
        )
        fallback_list = NovellLDAP.GetFallbackServerList(
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_SERVER", ""),
          server_list
        )
        Builtins.y2milestone("Primary server: %1", primary_server)
        Builtins.y2milestone("Fallback server_list: %1", fallback_list)

        full_admin_context = NovellUtils.Ldap2Nds(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        admin_pwd = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )

        if @xadMode
          xad_tree_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "NovellDns:XAD_TREE_ADMIN_CONTEXT:%1",
            xad_tree_admin_context
          )
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            full_admin_context = NovellUtils.Ldap2Nds(xad_tree_admin_context)
            admin_pwd = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_PASSWORD",
              ""
            )
          end
        end
        Builtins.y2milestone("NovellDns:admin:%1", full_admin_context)
        Builtins.y2milestone("NovellDns:pword:%1", admin_pwd) if @xadTestMode

        # Extend DNS Services schema.
        insufficient_rights = false
        schema_file = "/opt/novell/sch/DNIP.SCH"
        ndssch_file = "/opt/novell/eDirectory/bin/ndssch"
        cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(ndssch_file, " -h "),
                        Ops.get_string(
                          primary_server,
                          "CONFIG_LDAP_SERVER_IP",
                          ""
                        )
                      ),
                      " -F '"
                    ),
                    @schemaLogPath
                  ),
                  "' '"
                ),
                full_admin_context
              ),
              "'"
            ),
            " "
          ),
          schema_file
        )
        if @xadTestMode
          Builtins.y2milestone(
            "NovellDns:executing: %1",
            Ops.add(Ops.add(Ops.add("ADM_PASSWD='", admin_pwd), "' "), cmd)
          )
        else
          Builtins.y2milestone("novelldns:Executing %1", cmd)
        end

        rmap = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(Ops.add(Ops.add("ADM_PASSWD='", admin_pwd), "' "), cmd)
          )
        )
        if Ops.get_integer(rmap, "exit", -1) != 0
          if insufficient_rights_error(Ops.get_string(rmap, "stderr", "")) == true
            insufficient_rights = true
          end

          Builtins.y2milestone(
            "novelldns:Failed (error=%1) to extend schema using default IP address / hostname: %2, using file %3",
            Ops.get_integer(rmap, "exit", -1),
            Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
            schema_file
          )

          Builtins.foreach(fallback_list) do |server_info|
            cmd = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(ndssch_file, " -h "),
                            Ops.get_string(
                              server_info,
                              "CONFIG_LDAP_SERVER_IP",
                              ""
                            )
                          ),
                          " -F '"
                        ),
                        @schemaLogPath
                      ),
                      "' '"
                    ),
                    full_admin_context
                  ),
                  "'"
                ),
                " "
              ),
              schema_file
            )
            if @xadTestMode
              Builtins.y2milestone(
                "NovellDns:Retry:executing: %1",
                Ops.add(Ops.add(Ops.add("ADM_PASSWD='", admin_pwd), "' "), cmd)
              )
            end
            Builtins.y2milestone(
              "novelldns:Retry of schema extension with fallback IP address / hostname:Executing %1",
              cmd
            )
            rmap = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                Ops.add(Ops.add(Ops.add("ADM_PASSWD='", admin_pwd), "' "), cmd)
              )
            )
            if Ops.get_integer(rmap, "exit", -1) != 0
              if insufficient_rights_error(Ops.get_string(rmap, "stderr", "")) == true
                insufficient_rights = true
              end
              Builtins.y2milestone(
                "novelldns:Failed (error=%1) to extend schema using fallback IP address / hostname: %2, using file %3",
                Ops.get_integer(rmap, "exit", -1),
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                schema_file
              )
            else
              raise Break
            end
          end
        end
        if Ops.get_integer(rmap, "exit", -1) != 0
          if Mode.normal && insufficient_rights == true
            warn = Ops.add(
              Builtins.sformat(
                _("The user %1 has insufficient rights to extend schema.\n"),
                full_admin_context
              ),
              _(
                "This may not be a problem if joining a tree where the schema is already extended."
              )
            )

            Builtins.y2milestone("novelldns.write:%1", warn)
          end
          Builtins.y2milestone(
            "novelldns.write:Failed to extend schema for Novell DNS Services. Schema may already be extended if joining a NetWare Tree."
          )
        end

        # We need to replicate the schema changes across the tree so the product install below will work.
        ncpAdmin = NovellUtils.Typeful2Typeless(full_admin_context)

        cmd = Builtins.sformat(
          "/opt/novell/eDirectory/bin/ldapconfig -R -t %1 -p %2 -a %3",
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_TREE_NAME", ""),
          Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
          ncpAdmin
        )
        Builtins.y2milestone(
          "novelldns:executing: %1",
          Ops.add(cmd, " -w ****")
        )

        if @xadTestMode
          Builtins.y2milestone(
            "executing: %1",
            Ops.add(Ops.add(Ops.add(cmd, " -w '"), admin_pwd), "'")
          )
        end

        # WARNING - MUST REMOVE PASSWORD FROM COMMAND LINE.
        rmap = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(Ops.add(Ops.add(cmd, " -w '"), admin_pwd), "'")
          )
        )
        if Ops.get_integer(rmap, "exit", -1) != 0
          Builtins.y2milestone(
            "novelldns:Failed (error=%1) to replicate schema using IP address / hostname: %2. Standard out:%3",
            Ops.get_integer(rmap, "exit", -1),
            Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
            Ops.get_string(rmap, "stdout", "")
          )
        end

        Builtins.foreach(fallback_list) do |server_info|
          cmd = Builtins.sformat(
            "/opt/novell/eDirectory/bin/ldapconfig -R -t %1 -p %2 -a %3",
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_TREE_NAME", ""),
            Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
            ncpAdmin
          )
          Builtins.y2milestone(
            "novelldns:executing: %1",
            Ops.add(cmd, " -w ****")
          )
          if @xadTestMode
            Builtins.y2milestone(
              Ops.add(
                Ops.add(Ops.add(Ops.add("executing:", cmd), " -w '"), admin_pwd),
                "'"
              )
            )
          end
          rmap = Convert.to_map(
            SCR.Execute(
              path(".target.bash_output"),
              Ops.add(Ops.add(Ops.add(cmd, " -w '"), admin_pwd), "'")
            )
          )
          if Ops.get_integer(rmap, "exit", -1) != 0
            Builtins.y2milestone(
              "novelldns:Failed (error=%1) to replicate schema using IP address / hostname: %2. Standard out:%3",
              Ops.get_integer(rmap, "exit", -1),
              Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
              Ops.get_string(rmap, "stdout", "")
            )
          end
        end


        # Call the Novell DNS Services config module
        ldap_host = Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", "")
        ldap_port = Ops.get_integer(primary_server, "CONFIG_LDAP_PORT", 389)
        ldap_port = 1389 if @xadMode
        ldap_basedn = NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_BASEDN", "")
        )
        server_context = NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_DNS_SERVER_CONTEXT", "")
        )
        locator_context = NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_DNS_LOCATOR_CONTEXT", "")
        )
        group_context = NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_DNS_GROUP_CONTEXT", "")
        )

        runtimeAdmin = NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", "")
        )
        runtimePassword = Ops.get_string(
          @Settings,
          "CONFIG_DNS_LDAP_USER_PASSWORD",
          ""
        )

        useCommonProxy = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_DO_PROXY",
          ""
        )
        Builtins.y2milestone(
          "NovellDns.Write:useCommonProxy: %1",
          useCommonProxy
        )
        if useCommonProxy == "yes"
          commonProxyName = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
          )
          commonProxyPassword = NovellUtils.RetrieveProxyCredentials("password")
          if commonProxyPassword == nil || commonProxyPassword == ""
            commonProxyPassword = Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_PROXY_PASSWORD",
              ""
            )
          end

          if NovellUtils.Nds2Ldap(
              Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", "")
            ) == commonProxyName
            runtimePassword = commonProxyPassword
            Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_PASSWORD", runtimePassword)
          end
        end

        ssl = "0"
        ocs = "0"
        createServer = Ops.get_string(
          @Settings,
          "CONFIG_DNS_CREATE_SERVER_OBJECT",
          "no"
        )
        createServer = "no" if createServer == nil
        addServerParams = false
        addServerParams = true if Builtins.tolower(createServer) == "yes"
        hostName = Ops.get_string(@Settings, "CONFIG_DNS_HOST_NAME", "")
        domainName = Ops.get_string(@Settings, "CONFIG_DNS_DOMAIN_NAME", "")

        serverCreateParams = ""
        if addServerParams
          serverCreateParams = Ops.add(
            Ops.add(Ops.add(Ops.add("1 '", hostName), "' '"), domainName),
            "'"
          )
        end

        if Ops.get_string(@Settings, "CONFIG_DNS_STORE_CREDENTIAL_IN_OCS", "") == "yes"
          ocs = "1"
        end

        if Ops.get_string(@Settings, "CONFIG_DNS_USE_SECURE_PORT", "") != "no"
          ssl = "1"
          if @xadMode
            ldap_port = 1636
          else
            ldap_port = Ops.get_integer(
              primary_server,
              "CONFIG_LDAP_SECURE_PORT",
              636
            )
          end
        end

        cmd = Builtins.sformat(
          "/opt/novell/named/bin/dns-inst %1 %2 '%3' '---' '%4' '---' %5 '%6' '%7'",
          ldap_host,
          ldap_port,
          NovellUtils.Nds2Ldap(full_admin_context),
          runtimeAdmin,
          ocs,
          locator_context,
          group_context
        )


        cmd = Ops.add(
          cmd,
          Builtins.sformat(
            " '%1' '%2' %3 %4",
            ldap_basedn,
            server_context,
            serverCreateParams,
            ssl
          )
        )

        # y2milestone("mtc:dns.write:Executing %1 %2", reinstallStr, upgradeStr + reinstallStr + sformat("OES_DATA='%1' OES_DATA2='%2' %3", admin_pwd, runtimePassword, cmd));
        if @xadTestMode == false
          Builtins.y2milestone(
            "novelldns.write:Executing %1 %2",
            reinstallStr,
            cmd
          )
        else
          Builtins.y2milestone(
            "novelldns.write:Executing %1 %2",
            reinstallStr,
            Ops.add(
              Ops.add(upgradeStr, reinstallStr),
              Builtins.sformat(
                "OES_DATA='%1' OES_DATA2='%2' %3",
                admin_pwd,
                runtimePassword,
                cmd
              )
            )
          )
        end

        retval = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(
              Ops.add(upgradeStr, reinstallStr),
              Builtins.sformat(
                "OES_DATA='%1' OES_DATA2='%2' %3",
                admin_pwd,
                runtimePassword,
                cmd
              )
            )
          )
        )
        if Ops.get_integer(retval, "exit", -1) != 0
          ret = Ops.get_integer(retval, "exit", -1)
          errorMsg = Builtins.sformat(
            _("Unknown error while configuring OES DNS Services: %1"),
            ret
          )
          if ret == 1
            errorMsg = _(
              "BIND to LDAP server failed while configuring OES DNS Services."
            )
          elsif ret == 2
            errorMsg = _(
              "DNS Group object configuration failed while configuring OES DNS Services."
            )
          elsif ret == 3
            errorMsg = _(
              "DNS Locator object configuration failed while configuring OES DNS Services."
            )
          elsif ret == 4
            errorMsg = _(
              "DNS RootServerInfo object configuration failed while configuring OES DNS Services."
            )
          elsif ret == 5
            errorMsg = _(
              "NCP server object configuration for DNS failed while configuring OES DNS Services."
            )
          elsif ret == 6
            errorMsg = _(
              "DNS Locator Object configuration for trustee modification failed while configuring OES DNS Services."
            )
          elsif ret == 7
            errorMsg = _(
              "DNS RootServerInfo object configuration for trustee modification failed while configuring OES DNS Services."
            )
          elsif ret == 8
            errorMsg = Builtins.sformat(_("Could not create server object."))
          elsif ret == 9
            errorMsg = Builtins.sformat(_("NCP Server object search failed."))
          elsif ret == 10
            errorMsg = Builtins.sformat(_("DNS username creation failed."))
          end

          Builtins.y2error(errorMsg)
          Report.Error(errorMsg)
        end
      else
        if Mode.update || Mode.autoupgrade
          dns_proxyuser = NovellUtils.Nds2Ldap(Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", ""))
          proxyUser = NovellUtils.Nds2Ldap(
               Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))

          if dns_proxyuser == proxyUser
            Builtins.y2milestone("NovellDns:Setting commonproxy username and password during upgrade")
            proxyPassword = NovellUtils.RetrieveProxyCredentials("password")

            ret = NovellUtils.StoreCredsInOCSforUpgrade(proxyUser, proxyPassword,"dns-ldap")
            if ret != 0
              Popup.Message(
              Builtins.sformat(_("Error storing credentials in OCS for Common Proxy User: %1"),
              ret ))
              Builtins.y2milestone(
              "Novell-dns.ConfigureProxyUser:Error storing credentials in OCS for common Proxy User: %1",
              ret)
            end
          else
            Builtins.y2milestone("NovellDns:Dns using service proxy, moving to common proxy")
            NovellUtils.MovetoCommonProxyDuringUpgrade("novell-dns")
            Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
          end
        end
        if @upgradeMode || Mode.update || Mode.autoupgrade
          dns_proxyuser = NovellUtils.Nds2Ldap(Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", ""))
          proxyUser = NovellUtils.Nds2Ldap(
               Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
          if dns_proxyuser != proxyUser
            Builtins.y2milestone("NovellDns:Dns using service proxy, moving to common proxy")
            NovellUtils.MovetoCommonProxyDuringUpgrade("novell-dns")
            Ops.set(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
          end
        end
      end

      # Write the /etc/sysconfig file
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_LDAP_SERVER"),
        Builtins.sformat(
          "%1",
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_SERVER", "")
        )
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_LDAP_BASEDN"),
        NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_BASEDN", "")
        )
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_SERVER_CONTEXT"),
        NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_DNS_SERVER_CONTEXT", "")
        )
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_LOCATOR_CONTEXT"),
        NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_DNS_LOCATOR_CONTEXT", "")
        )
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_GROUP_CONTEXT"),
        NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_DNS_GROUP_CONTEXT", "")
        )
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_USE_SECURE_PORT"),
        Ops.get_string(@Settings, "CONFIG_DNS_USE_SECURE_PORT", "yes")
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_LDAP_USER_CONTEXT"),
        Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", "")
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_CREATE_SERVER_OBJECT"),
        Ops.get_string(@Settings, "CONFIG_DNS_CREATE_SERVER_OBJECT", "no")
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_HOST_NAME"),
        Ops.get_string(@Settings, "CONFIG_DNS_HOST_NAME", "")
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_DOMAIN_NAME"),
        Ops.get_string(@Settings, "CONFIG_DNS_DOMAIN_NAME", "")
      )
      SCR.Write(
        path(".sysconfig.novell-dns.CONFIG_DNS_STORE_CREDENTIAL_IN_OCS"),
        Ops.get_string(@Settings, "CONFIG_DNS_STORE_CREDENTIAL_IN_OCS", "")
      )

      # open ports 53 and 953
      NovellUtils.SetFirewallPort(53, :both, true)
      NovellUtils.SetFirewallPort(953, :both, true)
      if Mode.normal
        if Service.Status("boot.apparmor") == 0
          Service.Enable("boot.apparmor")
          Service.Reload("boot.apparmor")
          Builtins.y2milestone("inst_oes_postconfig:restarted apparmor")
        else
          Builtins.y2milestone(
            "inst_oes_postconfig:apparmor not running,profiles will not be reloaded"
          )
        end
      end

      if !@upgradeMode
        if Ops.get_integer(retval, "exit", -1) == 0
          SCR.Write(path(".sysconfig.novell-dns.SERVICE_CONFIGURED"), "yes")
          SCR.Write(
            path(".sysconfig.novell-dns.SERVICE_PREVIOUSLY_CONFIGURED"),
            "yes"
          )
        end
      else
        SCR.Write(path(".sysconfig.novell-dns.SERVICE_CONFIGURED"), "yes")
        SCR.Write(
          path(".sysconfig.novell-dns.SERVICE_PREVIOUSLY_CONFIGURED"),
          "yes"
        )
      end

      # Flush the sysconfig file.
      SCR.Write(path(".sysconfig.novell-dns"), nil)

      Builtins.y2debug("Novell DNS Services Settings(Write): %1", @Settings)

      return false if Abort()
      Progress.Finish
      Builtins.sleep(sl)

      return false if Abort()
      true
    end

    # Get all novell-dns settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "novell-dns", false)
        return true
      end

      Ops.set(
        @Settings,
        "CONFIG_DNS_LDAP_SERVER",
        Ops.get_string(settings, "ldap_server", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_DNS_LDAP_BASEDN",
        Ops.get_string(settings, "ldap_basedn", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_DNS_SERVER_CONTEXT",
        Ops.get_string(settings, "server_context", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_DNS_LOCATOR_CONTEXT",
        Ops.get_string(settings, "locator_context", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_DNS_GROUP_CONTEXT",
        Ops.get_string(settings, "group_context", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_DNS_USE_SECURE_PORT",
        Ops.get_string(settings, "use_secure_port", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_DNS_CREATE_SERVER_OBJECT",
        Ops.get_string(settings, "create_server_object", "no")
      )
      Ops.set(
        @Settings,
        "CONFIG_DNS_HOST_NAME",
        Ops.get_string(settings, "host_name", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_DNS_DOMAIN_NAME",
        Ops.get_string(settings, "domain_name", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_DNS_STORE_CREDENTIAL_IN_OCS",
        Ops.get_string(settings, "ocs_store", "yes")
      )
      @globals_configured = true
      true
    end

    # Dump the novell-dns settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-dns") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "novell-dns", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "novell-dns", true)
        Ops.set(
          settings,
          "ldap_server",
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_SERVER", "")
        )
        Ops.set(
          settings,
          "ldap_basedn",
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_BASEDN", "")
        )
        Ops.set(
          settings,
          "server_context",
          Ops.get_string(@Settings, "CONFIG_DNS_SERVER_CONTEXT", "")
        )
        Ops.set(
          settings,
          "locator_context",
          Ops.get_string(@Settings, "CONFIG_DNS_LOCATOR_CONTEXT", "")
        )
        Ops.set(
          settings,
          "group_context",
          Ops.get_string(@Settings, "CONFIG_DNS_GROUP_CONTEXT", "")
        )
        Ops.set(
          settings,
          "use_secure_port",
          Ops.get_string(@Settings, "CONFIG_DNS_USE_SECURE_PORT", "")
        )
        Ops.set(
          settings,
          "create_server_object",
          Ops.get_string(@Settings, "CONFIG_DNS_CREATE_SERVER_OBJECT", "no")
        )
        Ops.set(
          settings,
          "host_name",
          Ops.get_string(@Settings, "CONFIG_DNS_HOST_NAME", "")
        )
        Ops.set(
          settings,
          "domain_name",
          Ops.get_string(@Settings, "CONFIG_DNS_DOMAIN_NAME", "")
        )
        Ops.set(
          settings,
          "ocs_store",
          Ops.get_string(
            @Settings,
            "CONFIG_DNS_STORE_CREDENTIAL_IN_OCS",
            "yes"
          )
        )
      end

      deep_copy(settings)
    end

    # Dump all of the novell-dns settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def ExportSettings
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-dns") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "novell-dns", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "novell-dns", true)
        Ops.set(
          settings,
          "ldap_server",
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_SERVER", "")
        )
        Ops.set(
          settings,
          "ldap_basedn",
          Ops.get_string(@Settings, "CONFIG_DNS_LDAP_BASEDN", "")
        )
        Ops.set(
          settings,
          "server_context",
          Ops.get_string(@Settings, "CONFIG_DNS_SERVER_CONTEXT", "")
        )
        Ops.set(
          settings,
          "locator_context",
          Ops.get_string(@Settings, "CONFIG_DNS_LOCATOR_CONTEXT", "")
        )
        Ops.set(
          settings,
          "group_context",
          Ops.get_string(@Settings, "CONFIG_DNS_GROUP_CONTEXT", "")
        )
        Ops.set(
          settings,
          "use_secure_port",
          Ops.get_string(@Settings, "CONFIG_DNS_USE_SECURE_PORT", "")
        )
        Ops.set(
          settings,
          "create_server_object",
          Ops.get_string(@Settings, "CONFIG_DNS_CREATE_SERVER_OBJECT", "no")
        )
        Ops.set(
          settings,
          "host_name",
          Ops.get_string(@Settings, "CONFIG_DNS_HOST_NAME", "")
        )
        Ops.set(
          settings,
          "domain_name",
          Ops.get_string(@Settings, "CONFIG_DNS_DOMAIN_NAME", "")
        )
        Ops.set(
          settings,
          "ocs_store",
          Ops.get_string(
            @Settings,
            "CONFIG_DNS_STORE_CREDENTIAL_IN_OCS",
            "yes"
          )
        )
      end

      deep_copy(settings)
    end

    # Tranlate a yes or no string
    # @return translated "yes" or "no" string
    def Trans_yes_no(yesno)
      if Builtins.tolower(yesno) == "yes"
        return _("yes")
      else
        return _("no")
      end
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if @xadMode
        summary = Summary.OpenList(summary)

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          _(
            "DNS values are set and configured in eDirectory when installing DSFW. Configure will not occur in the DNS module."
          )
        )

        summary = Summary.CloseList(summary)

        Builtins.y2milestone("NovellDns.Summary: XAD mode.")
        return [summary, []]
      end

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("LDAP Server: %1"),
            Ops.get_string(@Settings, "CONFIG_DNS_LDAP_SERVER", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("LDAP base DN name with context: %1"),
            Ops.get_string(@Settings, "CONFIG_DNS_LDAP_BASEDN", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Server Context: %1"),
            Ops.get_string(@Settings, "CONFIG_DNS_SERVER_CONTEXT", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Locator Context: %1"),
            Ops.get_string(@Settings, "CONFIG_DNS_LOCATOR_CONTEXT", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Root Server Info Context: %1"),
            NovellUtils.Nds2Ldap(
              Ops.get_string(@Settings, "CONFIG_DNS_LDAP_BASEDN", "")
            )
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Group Context: %1"),
            Ops.get_string(@Settings, "CONFIG_DNS_GROUP_CONTEXT", "")
          )
        )


        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Proxy User for DNS Management: %1"),
            Ops.get_string(@Settings, "CONFIG_DNS_LDAP_USER_CONTEXT", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Store proxy credentials in OCS: %1"),
            Trans_yes_no(
              Ops.get_string(
                @Settings,
                "CONFIG_DNS_STORE_CREDENTIAL_IN_OCS",
                "yes"
              )
            )
          )
        )

        createServer = Ops.get_string(
          @Settings,
          "CONFIG_DNS_CREATE_SERVER_OBJECT",
          ""
        )
        createServer = "no" if createServer == nil

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Create server object: %1"),
            Trans_yes_no(createServer)
          )
        )

        if Builtins.tolower(createServer) == "yes"
          summary = Summary.AddListItem(
            summary, # Configuration summary text for autoyast
            Builtins.sformat(
              _("Host name: %1"),
              Ops.get_string(@Settings, "CONFIG_DNS_HOST_NAME", "")
            )
          )

          summary = Summary.AddListItem(
            summary, # Configuration summary text for autoyast
            Builtins.sformat(
              _("Domain name for DNS server: %1"),
              Ops.get_string(@Settings, "CONFIG_DNS_DOMAIN_NAME", "")
            )
          )
        end

        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :edir_service_configured, :type => "boolean"
    publish :variable => :xad_packages_installed, :type => "boolean"
    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :xadTestMode, :type => "boolean"
    publish :variable => :schemaLogPath, :type => "string"
    publish :variable => :Settings, :type => "map"
    publish :variable => :oldSettings, :type => "map"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :reinstallMode, :type => "boolean"
    publish :variable => :upgradeMode, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :GetTreeType, :type => "string ()"
    publish :function => :ServerIsDsfwServer, :type => "boolean (string)"
    publish :function => :GetContextInfo, :type => "list <string> (string)"
    publish :function => :ValidateCreds, :type => "boolean (string, string)"
    publish :function => :UserExists, :type => "boolean (string)"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :ExportSettings, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  NovellDns = NovellDnsClass.new
  NovellDns.main
end
